<?php

/**
 Plugin Name: WP Twitter Tweets
 Description: Using the TmhOAuth library, this plugin provides a widget and themeable function to output tweets from a given Twitter App.
 Version: 1.1.0
 Plugin URI: http://www.jnorton.co.uk
 Author: Justin Norton
 Author URI: http://www.jnorton.co.uk
 */

if (!class_exists('WP_Twitter_Tweets')) {

	class WP_Twitter_Tweets
	{

		var $settings;

		function WP_Twitter_Tweets()
		{
			date_default_timezone_set('UTC');
			require 'lib/tmhOAuth/tmhOAuth.php';
			require 'lib/tmhOAuth/tmhUtilities.php';
			//register_activation_hook(__FILE__, array(&$this, 'saveDefaultSettings'));
			$settings = $this->getSettings();
			$this->settings = $settings;
			add_action('admin_init', array(&$this, 'checkForSettingsSave'));
			add_action('admin_menu', array(&$this, 'addAdministrativeElements'));
			//custom actions
			add_action( 'wp_twitter_tweets_get_tweet', array(&$this, 'wp_twitter_tweets_get_tweet'));
		}

		//admin
		function addAdministrativeElements()
		{
			add_options_page(__('WP Twitter Tweets'), __('WP Twitter Tweets'), 'manage_options', 'wp-twitter-tweets', array(&$this, 'displaySettingsPage'));
		}

		function displaySettingsPage()
		{
			include 'lib/admin/settings.php';
		}

		function checkForSettingsSave()
		{
			if (isset($_POST['save-wp-twitter-tweets-settings']) && current_user_can('manage_options') && check_admin_referer('save-wp-twitter-tweets-settings')) {
				$settings = $this->getSettings();
				//general
				$settings['twitter_user_name'] = $_POST['twitter_user_name'];
				$settings['twitter_consumer_key'] = $_POST['twitter_consumer_key'];
				$settings['twitter_consumer_secret'] = $_POST['twitter_consumer_secret'];
				$settings['oauth_access_token'] = $_POST['oauth_access_token'];
				$settings['oauth_access_token_secret'] = $_POST['oauth_access_token_secret'];
				$settings['max_fetch_count'] = $_POST['max_fetch_count'];
				$this->saveSettings($settings);
				wp_redirect(admin_url('options-general.php?page=wp-twitter-tweets&updated=1'));
			}
		}

		function saveDefaultSettings()
		{
			$settings = $this->getSettings();
			if ( empty($settings)) {
				$settings = array();
				//general
				$settings['twitter_user_name'] = "";
				$settings['twitter_consumer_key'] = "";
				$settings['twitter_consumer_secret'] = "";
				$settings['oauth_access_token'] = "";
				$settings['oauth_access_token_secret'] = "";
				$settings['max_fetch_count'] = 3200;
				$this->saveSettings($settings);
			}
		}

		function getSettings()
		{
			if ($this->settings === null) {
				$defaults = array();
				$this->settings = get_option('WP Twitter Tweets Settings', array());
			}
			return $this->settings;
		}


		function saveSettings($settings)
		{
			if (!is_array($settings)) {
				return;
			}
			$this->settings = $settings;
			update_option('WP Twitter Tweets Settings', $this->settings);
		}


		function wp_twitter_tweets_get_tweet($vars)
		{

			$defaults = array(
				"item_number"=>false,
				"display_count"=>10,	
				"wrapper"=>false,
				"hashtag"=>false,
				"referrer"=>"_theme",
				"enable_cache"=>false,
				"cache_expiry"=>60,
			);

			foreach ($vars as $key => $value) {
				if ($value != '' || !isset($vars[$key])) {
					continue;
				}
				$vars[$key] = $defaults[$key];
			}

			extract($vars);

			$settings = $this->settings;
			if ($enable_cache == true) {
				$tweetTransient = get_transient( 'twitter_api_result'.$referrer );
				
			} else {
				$tweetTransient == false;
			}

			if ($tweetTransient == false ) {

				$consumer_key = $settings['twitter_consumer_key'];
				$consumer_secret = $settings['twitter_consumer_secret'];
				$oauth_access_token = $settings['oauth_access_token'];
				$oauth_access_token_secret = $settings['oauth_access_token_secret'];

				$tmhOAuth = new tmhOAuth(array(
						'consumer_key' => $consumer_key,
						'consumer_secret' => $consumer_secret,
						'user_token' => $oauth_access_token,
						'user_secret' => $oauth_access_token_secret,
						'curl_ssl_verifypeer'   => false
					));

				$code = $tmhOAuth->request('GET', $tmhOAuth->url('1.1/statuses/user_timeline'), array(
						'include_entities' => '1',
						'include_rts' => '1',
						'screen_name' => $settings['twitter_user_name'],
						'count' => $count,
					));

				if ($code == 200) {
					$timeline = json_decode($tmhOAuth->response['response'], true);

					//print_r($timeline);

					$tweetArray = array();
					foreach ($timeline as $tweet) {
						$entified_tweet = tmhUtilities::entify_with_options($tweet);
						$is_retweet = isset($tweet['retweeted_status']);

						$diff = time() - strtotime($tweet['created_at']);
						if ($diff < 60*60)
							$created_at = floor($diff/60) . ' minutes ago';
						elseif ($diff < 60*60*24)
							$created_at = floor($diff/(60*60)) . ' hours ago';
						else
							$created_at = date('d M', strtotime($tweet['created_at']));

						if ($hashtag == false) {

							$permalink = str_replace(
								array(
									'%screen_name%',
									'%id%',
									'%created_at%',
									'%tweet_text%'
								),
								array(
									$tweet['user']['screen_name'],
									$tweet['id_str'],
									$created_at,
									$tweet['text'],
								),
								'<a href="https://twitter.com/%screen_name%/" target="_blank">%tweet_text%</a>'
							);

							$tweetArray[] = $permalink;

						} else {

							foreach ($tweet['entities']['hashtags'] as $hashtags) {

								foreach ($hashtags as $key => $hash) {
									
									if ($hashtag == $hash) {
										$permalink = str_replace(
											array(
												'%screen_name%',
												'%id%',
												'%created_at%',
												'%tweet_text%'
											),
											array(
												$tweet['user']['screen_name'],
												$tweet['id_str'],
												$created_at,
												$tweet['text'],
											),
											'<a href="https://twitter.com/%screen_name%/" target="_blank">%tweet_text%</a>'
										);

										$tweetArray[] = $permalink;
									}
								}

							}



						}

					}

					$tweetTransient = set_transient('twitter_api_result'.$referrer, $tweetArray, 60*$cache_expiry);
					if ($item_number != false) {
						if ($wrapper != false) {
							echo '<'.$wrapper.'>'.$tweetArray[$item_number].'</'.$wrapper.'>';
						} else {
							echo $tweetArray[$item_number];
						}

					} else {
						foreach ($tweetArray as $tweet) {
							
							if($counter >= $display_count){
								break;
							}
							$counter++;
							
							if ($wrapper != false) {
								echo '<'.$wrapper.'>'.$tweet.'</'.$wrapper.'>';
							} else {
								echo $tweet;
							}
						}

					}
				} else {
					tmhUtilities::pr($tmhOAuth->response);
				}
			} else {

				if ($item_number != false) {
					if ($wrapper != false) {
						echo '<'.$wrapper.'>'.$tweetTransient[$item_number].'</'.$wrapper.'>';
					} else {
						echo $tweetTransient[$item_number];
					}
				} else {

					foreach ($tweetTransient as $tweet) {
					
							if($counter >= $display_count){
								break;
							}
							$counter++;
							
						if ($wrapper != false) {
							echo '<'.$wrapper.'>'.$tweet.'</'.$wrapper.'>';
						} else {
							echo $tweet;
						}

					}

				}

			}
		}

	}

	$WP_Twitter_Tweets = new WP_Twitter_Tweets();

	//widgets
	if ( !function_exists( 'WP_Twitter_Tweets_Widget' ) ) {
		function WP_Twitter_Tweets_Widget()
		{
			register_widget("WP_Twitter_Tweets_Widget");
		}
	}
	add_action('widgets_init', 'WP_Twitter_Tweets_Widget');

	//CUSTOM TEXT WIDGET
	class WP_Twitter_Tweets_Widget extends WP_Widget {

		var $WP_Twitter_Tweets_Settings;

		function WP_Twitter_Tweets_Widget()
		{
			$this->WP_Widget('WP_Twitter_Tweet', 'WP Twitter Tweets Widget');
		}

		function update($instance, $old_instance)
		{
			delete_transient( 'twitter_api_result_widget' );
			return $instance;
		}
		function form($instance)
		{
			global $WP_Twitter_Tweets;
			$WP_Twitter_Tweets_settings = $WP_Twitter_Tweets->getSettings();
			$instance = wp_parse_args((array) $instance, array(
					'twitter_user_name' => $WP_Twitter_Tweets_settings['twitter_user_name'],
					'twitter_consumer_key' => $WP_Twitter_Tweets_settings['twitter_consumer_key'],
					'twitter_consumer_secret' => $WP_Twitter_Tweets_settings['twitter_consumer_secret'],
					'oauth_access_token' => $WP_Twitter_Tweets_settings['oauth_access_token'],
					'oauth_access_token_secret' => $WP_Twitter_Tweets_settings['oauth_access_token_secret'],
					'item_number' => '',
					'hashtag' => '',
					'display_count' => '',
					'wrapper' => '',
					'enable_cache' => '',
				));

			//Defaults
			# hash tag
			$output .= '<p><label for="' . $this->get_field_name('hashtag') . '">';
			$output .= 'Hash Tag:</label>';
			$output .= '<input class="widefat"';
			$output .= 'id="' . $this->get_field_id('hashtag') . '"';
			$output .= 'name="' . $this->get_field_name('hashtag') . '"';
			$output .= 'type="text"';
			$output .= 'value="' . esc_attr($instance["hashtag"]) . '" />';

			# item_number
			$output .= '<p><label for="' . $this->get_field_name('item_number') . '">';
			$output .= 'Item Number:</label>';
			$output .= '<input class="widefat"';
			$output .= 'id="' . $this->get_field_id('item_number') . '"';
			$output .= 'name="' . $this->get_field_name('item_number') . '"';
			$output .= 'type="text"';
			$output .= 'value="' . esc_attr($instance["item_number"]) . '" />';

			# count
			$output .= '<p><label for="' . $this->get_field_name('display_count') . '">';
			$output .= 'Maximum Number of Tweets to Display:</label>';
			$output .= '<input class="widefat"';
			$output .= 'id="' . $this->get_field_id('display_count') . '"';
			$output .= 'name="' . $this->get_field_name('display_count') . '"';
			$output .= 'type="text"';
			$output .= 'value="' . esc_attr($instance["display_count"]) . '" />';

			# wrapper
			$output .= '<p><label for="' . $this->get_field_name('wrapper') . '">';
			$output .= 'HTML Wrapper e.g. strong, em, li:</label>';
			$output .= '<input class="widefat"';
			$output .= 'id="' . $this->get_field_id('wrapper') . '"';
			$output .= 'name="' . $this->get_field_name('wrapper') . '"';
			$output .= 'type="text"';
			$output .= 'value="' . esc_attr($instance["wrapper"]) . '" />';

			# cache
			$output .= '<p><label for="' . $this->get_field_name('enable_cache') . '">';
			$output .= 'Enable Cache:</label>';
			$output .= '<input type="checkbox" class="checkbox"';
			$output .= 'id="' . $this->get_field_id('enable_cache') . '"';
			$output .= 'name="' . $this->get_field_name('enable_cache') . '"';
			$output .= checked( (bool) $instance["enable_cache"], true, false );
			$output .= '" />';

			# cache expiry
			$output .= '<p><label for="' . $this->get_field_name('cache_expiry') . '">';
			$output .= 'Cache Expiry (minutes):</label>';
			$output .= '<input class="widefat"';
			$output .= 'id="' . $this->get_field_id('cache_expiry') . '"';
			$output .= 'name="' . $this->get_field_name('cache_expiry') . '"';
			$output .= 'type="text"';
			$output .= 'value="' . esc_attr($instance["cache_expiry"]) . '" />';

			echo $output;
		}

		function widget($args, $instance)
		{
			extract( $args );
			$vars = array(
				"item_number"=>$instance["item_number"],
				"display_count"=>$instance["display_count"],
				"wrapper"=>$instance["wrapper"],
				"hashtag"=>$instance["hashtag"],
				"referrer"=>$widget_id,
				"enable_cache"=>$instance["enable_cache"],
				"cache_expiry"=>$instance["cache_expiry"],
			);
			do_action('wp_twitter_tweets_get_tweet', $vars);
		}

	}


}


?>
